#!/usr/bin/env python3

"""
Prints :math:`\\operatorname{sd}^r(n) := (\\underbrace{\\operatorname{sd} \\circ
\\operatorname{sd} \\circ \\cdots \\circ \\operatorname{sd}}_{r})(n) =
\\underbrace{\\operatorname{sd}(\\operatorname{sd}(\\dots\\operatorname{sd}}_{r}(n)\\dots))`.
"""

def sd(n):
    """
    Returns the sum of digits of a natural number `n`.
    Used as an auxiliary function to `sdr`.
    """
    res = 0
    while n > 0:
        res += n % 10
        n //= 10
    return res

def sdr(n, r = None):
    """
    If `r <= 0`, returns `n`.
    If `r > 0` returns `sd(sd(...sd(n)...))`, where `sd` is applied `r` times.
    If `r` is undefined, returns the fixed point of `sd(sd(...sd(n)...))`, which
    is the first single-digit number that this composition encounters.
    """
    n = abs(n)
    # r not defined
    if r is None:
        while n >= 10:
            n = sd(n)
        return n
    # r non-positive
    if r <= 0:
        return n
    # r positive
    for _ in range(r):
        n = sd(n)
    return n

def main():
    n = 123
    r = 3
    print("sdr(" + str(n) + ", " + str(r) + ") =", sdr(n, r))
    print("sdr(" + str(n) + ") =", sdr(n))

main()

